# JHC - Jabber Humane Clock.
# Copyright (C) 2007  Denis Nelubin aka Gelin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# http://gelin.ru
# mailto:den@gelin.ru
# xmpp:gelin@jabber.ru?message

""" This module provides Dictionary class which represents dictionary for
    time values. Each hour/minute pair has it"s own (may be many) string
    representation. They are saved in SQLite database.
"""

import logging
import sqlite3 as db
import time
import random
import types

class Dictionary:
    """ Gets time (hour + minute) string description from SQLite database"""

    def __init__(self, dbName):
        """ Creates dictionary, specified database file will be used
            to get time strings
        """
        self.sql = "SELECT string FROM time_string WHERE hour = ? AND minute = ?;"
        self.log = logging.getLogger("jhc.dictionary.Dictionary")
        try:
            self.connection = db.connect(database=dbName)
        except db.DatabaseError, x:
            self.log.error("Can't connect to database " + dbName, exc_info=x)

    def getTime(self, seconds):
        """ Returns tuple of (hour, minute) from time specified as number
            (as it"s returned by time.time() function).
        """
        return time.localtime(seconds)[3:5]

    def getStrings(self, hourMinute):
        """ Returns tuple of strings for specified hour and minute """
        result = []
        if self.connection:
            try:
                cursor = self.connection.cursor()
                cursor.execute(self.sql, (hourMinute[0], hourMinute[1]))
                for row in cursor.fetchall():
                    result.append(row[0])
                cursor.close()
            except db.DatabaseError, x:
                self.log.error("Can't get string for " + str(hourMinute), exc_info=x)
        else:
            self.log.error("Connection to database was not created")
        return result

    def chooseString(self, strings):
        """ Randomly chooses one string from provided list """
        if len(strings) > 0:
            return strings[random.randint(0, len(strings) - 1)]
        else:
            return None

    def getString(self, seconds):
        """ Main method, returns string representation of specified time
            Second parameter can be number of seconds from Epoch or
            (hour, minute) tuple """
        if seconds == None:
            seconds = time.time()
        elif isinstance(seconds, types.TupleType):
            (hour, minute) = seconds
        else:
            (hour, minute) = self.getTime(seconds)
        string = self.chooseString(self.getStrings((hour, minute)))
        if string == None:
            string = "%02i:%02i" % (hour, minute)
        return string