"""Tests for jhc.bot."""

import unittest

import xmpp
import optparse

import jhc.bot


class BotTest(unittest.TestCase):

    def setUp(self):
        self.options = optparse.Values()
        self.options.database = "../data/test.db"
        self.options.user = "jhc"
        self.options.server = "jabber.localdomain"
        self.options.port = 5222
        self.options.password = "123"
        self.options.secure = None
        self.options.conferences = []
        self.options.noRoster = True
        self.options.proxy = {}
        self.options.time = (0, 0)
        self.options.interval = 30

    def tearDown(self):
        pass

    def testInitSuccess(self):
        bot = jhc.bot.Bot(self.options)
        self.assertTrue(bot.dictionary != None)

    def testConnect(self):
        bot = jhc.bot.Bot(self.options)
        bot.connect()
        self.assertTrue(bot.connection != None)
        #how to test connection to external system?

    def testMakeReplyMessage(self):
        bot = jhc.bot.Bot(self.options)
        message = xmpp.Message("to", "body", "typ", "subject", frm="from")
        message.setThread("thread")
        message = bot.makeReplyMessage(message)
        self.assertEqual("to", message.getFrom())
        self.assertEqual("from", message.getTo())
        self.assertEqual("typ", message.getType())
        self.assertEqual("subject", message.getSubject())
        self.assertEqual("thread", message.getThread())

    def testMakeFullJid(self):
        bot = jhc.bot.Bot(self.options)
        self.assertEqual("jhc@jabber.localdomain/resource",
                bot.makeFullJid("jhc@jabber.localdomain", "resource"))

    def testIsJoined(self):
        bot = jhc.bot.Bot(self.options)
        self.assertEqual(False, bot.isJoined("test@conference.jabber.localdomain"))
        bot.joinedConferences.append(xmpp.JID("test@conference.jabber.localdomain"))
        self.assertEqual(True, bot.isJoined("test@conference.jabber.localdomain/nick"))

    def testIsInvitationGroupchat(self):
        bot = jhc.bot.Bot(self.options)
        message = xmpp.Message(frm="test@jabber.localdomain/invitor")
        message.addChild(name="x", namespace="jabber:x:conference",
                attrs={"jid": "test@conference.jabber.localdomain"})
        self.assertEqual("test@conference.jabber.localdomain", bot.getInvitation(message))

    def testIsInvitationMUC(self):
        bot = jhc.bot.Bot(self.options)
        message = xmpp.Message(frm="test@conference.jabber.localdomain")
        x = message.addChild(name="x", namespace="http://jabber.org/protocol/muc#user")
        x.addChild(name="invite")
        self.assertEqual("test@conference.jabber.localdomain", bot.getInvitation(message))


def test_suite():
    return unittest.makeSuite(BotTest)

if __name__ == "__main__":
    unittest.main(defaultTest="test_suite")