#-*- coding: UTF-8 -*-
"""Tests for jhc.dictionary."""

import unittest

import time
import jhc.dictionary


class DictionaryTest(unittest.TestCase):

    def setUp(self):
        self.dictionary = jhc.dictionary.Dictionary("../data/test.db")

    def tearDown(self):
        self.dictionary = None

    def testGetTime(self):
        t = time.mktime((0, 0, 0, 1, 2, 0, 0, 0, 0))  #01:02
        self.assertEqual((1, 2), self.dictionary.getTime(t))

    def testGetStringsZero(self):
        self.assertEqual([], self.dictionary.getStrings((0, 0)))

    def testGetStringsOne(self):
        self.assertEqual([u"One hour"], self.dictionary.getStrings((1, 0)))

    def testGetStringsTwo(self):
        self.assertEqual([u"Two hours", u"Two hours and zero minutes"], self.dictionary.getStrings((2, 0)))

    def testChooseString(self):
        self.assert_(self.dictionary.chooseString(["one", "two"]) in ["one", "two"])

    def testChooseStringEmpty(self):
        self.assertEqual(None, self.dictionary.chooseString([]))

    def testGetStringZero(self):
        t = time.mktime((0, 0, 0, 0, 0, 0, 0, 0, 0))   #00:00
        self.assertEqual("00:00", self.dictionary.getString(t));

    def testGetStringOne(self):
        t = time.mktime((0, 0, 0, 1, 0, 0, 0, 0, 0))   #01:00
        self.assertEqual(u"One hour", self.dictionary.getString(t))

    def testGetStringTwo(self):
        t = time.mktime((0, 0, 0, 2, 0, 0, 0, 0, 0))   #01:00
        self.assert_(self.dictionary.getString(t) in [u"Two hours", u"Two hours and zero minutes"])

    def testGetStringByTuple(self):
        self.assertEqual(u"One hour", self.dictionary.getString((1, 0)))

    def testGetStringI18n(self):
        self.assertEqual(u"Три часа ночи", self.dictionary.getString((3, 0)))


def test_suite():
    return unittest.makeSuite(DictionaryTest)

if __name__ == "__main__":
    unittest.main(defaultTest="test_suite")