# JHC - Jabber Humane Clock.
# Copyright (C) 2007  Denis Nelubin aka Gelin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# http://gelin.ru
# mailto:den@gelin.ru
# xmpp:gelin@jabber.ru?message

""" This module provides Timer class which periodically calls
    specified function.
"""

import logging
import time
import threading

class Timer(threading.Thread):
    """ Periodically calls specified function.
    """

    def __init__(self, hourMinute, interval, function, args=[], kwargs={}):
        """ Creates timer.
            hourMinute - tuple with hour and minute, start time for function calls
            interval - call interval in minutes
            function - function to call
            args, kwargs - function arguments
        """
        threading.Thread.__init__(self, name="jhc.timer.Timer")
        self.setDaemon(True)
        self.log = logging.getLogger("jhc.timer.Timer")
        self.startMinute = hourMinute[0] * 60 + hourMinute[1]
        self.interval = interval
        self.function = function
        self.args = args
        self.kwargs = kwargs
        self.event = threading.Event()

    def run(self):
        """ Wraper for called functions. Makes a call and reinits timer.
        """
        while True:
            self.sleep()
            self.function(self.args, self.kwargs)

    def sleep(self):
        """ Sleep until time to next call become.
        """
        interval = self.getInterval()
        self.log.debug("interval = %s", interval)
        self.event.wait(interval)

    def getMinutes(self, seconds):
        """ Gets number of minutes from beginning of a day and a seconds part
            for specified number of seconds from beginning of epoch
            (got from time.time() for example).
        """
        (hour, minute, second) = time.localtime(seconds)[3:6]
        return (hour * 60 + minute, second)

    def getInterval(self):
        """ Calculates interval to next function call time in seconds.
        """
        seconds = time.time()
        (minute, second) = self.getMinutes(seconds)
        minutesShift = (self.startMinute - minute) % self.interval
        return minutesShift * 60 + (60 - second)